package cat.hack3.codingtests.marsrover;

import cat.hack3.codingtests.marsrover.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.cartography.Direction;
import cat.hack3.codingtests.marsrover.cartography.MarsMap;

import java.util.logging.Logger;

public class MarsRover {
    Logger logger = Logger.getLogger(this.getClass().getName());
    private enum Rotation {LEFT, RIGHT}

    private final MarsMap marsMap;
    private Direction currentDirection;

    public MarsRover(MarsMap marsMap, Direction startingDirection) {
        this.marsMap = marsMap;
        currentDirection = startingDirection;
    }

    public Coordinates moveForward() {
        return marsMap.updatePositionTowards(currentDirection);
    }

    public Coordinates moveBackwards() {
        return marsMap.updatePositionTowards(currentDirection.reversed());
    }

    public Direction turnLeft() {
        currentDirection = currentDirection.getNextDirectionRotatingToLeft();
        reportNewDirection(Rotation.LEFT);
        return currentDirection;
    }

    public Direction turnRight() {
        currentDirection = currentDirection.getNextDirectionRotatingToRight();
        reportNewDirection(Rotation.RIGHT);
        return currentDirection;
    }

    private void reportNewDirection(Rotation rotation) {
        logger.info(String.format("Rotated towards %s, now the direction is %s",
                rotation, currentDirection));
    }

    public Coordinates getCurrentCoordinates() {
        return marsMap.getCurrentPosition();
    }

    public Direction getCurrentDirection() {
        return currentDirection;
    }
}
