package cat.hack3.codingtests.marsrover;

import cat.hack3.codingtests.marsrover.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.cartography.Direction;
import cat.hack3.codingtests.marsrover.cartography.MarsMap;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;

import static cat.hack3.codingtests.marsrover.cartography.Direction.SOUTH;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotEquals;

public class MarsRoverWithObstaclesTest {

    private MarsRover rover;

    @BeforeMethod
    public void setUp() {
        int mapWidth = 10;
        int mapHeight = 10;
        int latitudeStartingPoint = 2;
        int longitudeStartingPoint = 3;
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);
        List<Coordinates> obstaclesLocalizations = List.of(
                Coordinates.of(3, 3),
                Coordinates.of(5, 5),
                Coordinates.of(7, 7),
                Coordinates.of(9, 9),
                Coordinates.of(10, 10)
        );
        var marsMap = new MarsMap(mapHeight, mapWidth, startingCoordinates, obstaclesLocalizations);
        rover = new MarsRover(marsMap, SOUTH);
    }

    @Test
    public void findAnObstacleAtFirstMove() {
        Coordinates originalCoordinates = rover.getCurrentCoordinates();
        Direction originalDirection = rover.getCurrentDirection();

        rover.moveForward();

        assertEquals(rover.getCurrentCoordinates(), originalCoordinates);
        assertEquals(rover.getCurrentDirection(), originalDirection);
    }

    @Test
    public void afterFindingObstacleWalkAroundUntilFindingNext() {
        Coordinates oneStepForward = rover.moveForward();
        assertNotEquals(oneStepForward, Coordinates.of(3, 3));

        rover.turnLeft();
        rover.moveForward();
        rover.moveForward();
        rover.turnRight();
        rover.moveForward();
        rover.moveForward();
        var currentPosition = Coordinates.of(4, 5);
        assertEquals(rover.getCurrentCoordinates(), currentPosition);
        rover.moveForward();
        var positionAfterMove = Coordinates.of(5, 4);
        assertNotEquals(rover.getCurrentCoordinates(), positionAfterMove);
    }

}
